import moment from 'moment-timezone'

/**
 * Waits for the element until it is present in the DOM
 *
 * @param { String } selector - particular node selector
 * @returns { Object } - resolved or rejected promise
 */
export const wait_for_element = selector => {
    return new Promise(resolve => {
        // Return element if it is present
        const element = document.querySelector(selector)
        if (element) {
            resolve(element)
            return
        }

        // Otherwise observe for another mutation
        new MutationObserver((_, observer) => {
            for (const element of document.querySelectorAll(selector)) {
                resolve(element)
                observer.disconnect()
            }
        }).observe(document.documentElement, { childList: true, subtree: true })
    })
}

export const get_tref_class = (className) => `tref-pain-${className}`
export const get_tref_func = (prefix) => (className) => get_tref_class(`${prefix}-${className}`)

export const create_params = (paramsObject) => {
    const urlSearchParams = new URLSearchParams()

    Object.entries(paramsObject).forEach(([key, value]) => {
        if (value === undefined || value === null || value === '') {
            return
        } else if (Array.isArray(value)) {
            value.forEach((val) => urlSearchParams.append(key, val))
        } else {
            urlSearchParams.set(key, value)
        }
    })

    return urlSearchParams
}

export const options_to_params = (options) => {
    const paramsObject = {
        limit: options.itemsPerPage === -1 ? undefined : options.itemsPerPage,
        offset: (options.page - 1) * options.itemsPerPage,
        order_by: options.sortBy[0] ? `${options.sortDesc[0] ? '-' : '+'}${options.sortBy[0]}` : undefined,
    }

    return create_params(paramsObject)
}

export const join_params = (param_1, param_2) => {
    const params = new URLSearchParams(param_1)
    param_2.forEach((value, key) => params.append(key, value))
    return params
}

export const date_range_to_from_to = (date_range) => {
    const from = moment().startOf(date_range.type).add(date_range.offset, date_range.type)
    const to = moment(from).add(1, date_range.type)
    return { from: from.format(), to: to.format() }
}

export const export_to_csv = (headers, rows, handle, row_to_array) => {
    const csv = [headers.join(';') + '\n']

    for (const row of rows) {
        csv.push(row_to_array(row).join(';') + '\n')
    }

    const blob = new Blob(csv, { type: 'text/csv;charset=utf-8' })
    const url = URL.createObjectURL(blob)

    const link = document.createElement('a')
    link.setAttribute('href', url)
    link.setAttribute('download', `${handle}-${moment().format('YYYY-MM-DDTHH-mm-ss')}.csv`)
    link.click()

    URL.revokeObjectURL(url)
}
